@@ -31,6 +31,9 @@ When doing sorting ATTRIBUTE_SORT_KEY is consulted to canonicalize the
 value. See tiddlyweb.filters.sort.
 """
 
+from itertools import ifilter
+from operator import gt, lt
+
 from tiddlyweb.model.tiddler import Tiddler
 from tiddlyweb.store import NoTiddlerError
 from tiddlyweb.filters.sort import ATTRIBUTE_SORT_KEY
@@ -49,6 +52,10 @@ def select_parse(command):
         args = args.replace('!', '', 1)
 
         def selector(entities, indexable=False, environ=None):
+            """
+            Perform a negated select: match when attribute and value are not
+            equal.
+            """
             return select_by_attribute(attribute, args, entities, negate=True,
                     environ=environ)
 
@@ -56,6 +63,10 @@ def select_parse(command):
         args = args.replace('<', '', 1)
 
         def selector(entities, indexable=False, environ=None):
+            """
+            Perform a less than select: match when attribute is less than
+            value.
+            """
             return select_relative_attribute(attribute, args, entities,
                     lesser=True, environ=environ)
 
@@ -63,12 +74,19 @@ def select_parse(command):
         args = args.replace('>', '', 1)
 
         def selector(entities, indexable=False, environ=None):
+            """
+            Perform a greater than select: match when attribute is greater than
+            value.
+            """
             return select_relative_attribute(attribute, args, entities,
                     greater=True, environ=environ)
 
     else:
 
         def selector(entities, indexable=False, environ=None):
+            """
+            Perform a match select: match when attribute is equal the value.
+            """
             if environ == None:
                 environ = {}
             return select_by_attribute(attribute, args, entities,
@@ -81,7 +99,7 @@ def bag_in_recipe(entity, attribute, value):
     """
     Return true if the named bag is in the recipe.
     """
-    bags = [bag for bag, filter in entity.get_recipe()]
+    bags = [bag for bag, _ in entity.get_recipe()]
     return value in bags
 
 
@@ -154,19 +172,28 @@ def select_by_attribute(attribute, value, entities, negate=False,
         imported_module = __import__(indexer, {}, {}, ['index_query'])
         # dict keys may not be unicode
         kwords = {str(attribute): value, 'bag': indexable.name}
-        for tiddler in imported_module.index_query(environ, **kwords):
-            yield tiddler
+        return imported_module.index_query(environ, **kwords)
     else:
         select = ATTRIBUTE_SELECTOR.get(attribute, default_func)
-        for entity in entities:
+
+        def _posfilter(entity):
+            """
+            Return True if the entity's attribute matches value.
+            """
             stored_entity = _get_entity(entity, store)
-            if negate:
-                if not select(stored_entity, attribute, value):
-                    yield entity
-            else:
-                if select(stored_entity, attribute, value):
-                    yield entity
-    return
+            return select(stored_entity, attribute, value)
+
+        if negate:
+            def _negfilter(entity):
+                """
+                Return True if the entity's attribute does not match value.
+                """
+                return not _posfilter(entity)
+            _filter = _negfilter
+        else:
+            _filter = _posfilter
+
+        return ifilter(_filter, entities)
 
 
 def select_relative_attribute(attribute, value, entities,
@@ -189,26 +216,27 @@ def select_relative_attribute(attribute, value, entities,
 
     func = ATTRIBUTE_SORT_KEY.get(attribute, normalize_value)
 
-    for entity in entities:
+    if greater:
+        comparator = gt
+    elif lesser:
+        comparator = lt
+    else:
+        comparator = lambda x, y: True
+
+    def _select(entity):
+        """
+        Return true if entity's attribute is < or > (depending on
+        comparator) the value in the filter.
+        """
         stored_entity = _get_entity(entity, store)
-        if greater:
-            if hasattr(stored_entity, 'fields'):
-                if func(getattr(stored_entity, attribute, stored_entity.fields.get(
-                    attribute, None))) > func(value):
-                    yield entity
-            else:
-                if func(getattr(stored_entity, attribute, None)) > func(value):
-                    yield entity
-        elif lesser:
-            if hasattr(stored_entity, 'fields'):
-                if func(getattr(stored_entity, attribute, stored_entity.fields.get(
-                    attribute, None))) < func(value):
-                    yield entity
-            else:
-                if func(getattr(stored_entity, attribute, None)) < func(value):
-                    yield entity
+        if hasattr(stored_entity, 'fields'):
+            return comparator(func(getattr(stored_entity, attribute,
+                stored_entity.fields.get(attribute, None))), func(value))
         else:
-            yield entity
+            return comparator(func(getattr(stored_entity, attribute, None)),
+                    func(value))
+
+    return ifilter(_select, entities)
 
 
 def _get_entity(entity, store):
